/* $Id: swig-ruby.h,v 1.8 2003/10/28 11:24:59 mackstann Exp $ */

// This file is not meant to be used alone, only from wrapper.i
// and wrapper.i's output

//
// This file contains C++ code needed to work around swig's lack of support for
// various features of ruby.  Namely automatic handling of multiple inheritance
// in C++.

#ifndef SWIG
#define K_SWIG_WRAPPER(name, cls) SWIG_NewPointerObj((void*)name, SWIGTYPE_p_##cls, 1)
#endif

#ifdef SWIG
%feature("notabstract") RubyScreenFunction;
%feature("notabstract") RubySimpleFunction;
#endif

class RubyFunction {
private:
    VALUE rb_meth;
protected:
    inline void call(VALUE args) {
        rb_apply(rb_meth, rb_intern("call"), args);
                //TODO:  Do stuff to call the ruby method with the 
                // args supplied
    }
public:
    RubyFunction(VALUE a_method) {
        rb_meth = a_method;
    }
};

class RubyMenuFunction: public RubyFunction, public MenuFunction {
#ifdef SWIG
    %mixin RubyScreenFunction "Kahakai::MenuFunction";
#endif
public:
    RubyMenuFunction(VALUE meth): RubyFunction(meth) {};

    virtual void operator()(KaMenuItem* mi, XEvent* event, 
                                    KaAction* action) {
                VALUE args = rb_ary_new3(3, 
                                         K_SWIG_WRAPPER(mi, KaMenuItem),
                                         K_SWIG_WRAPPER(event,  XEvent), 
                                         K_SWIG_WRAPPER(action, KaAction));
                call(args);
    }
};

class RubyScreenFunction: public RubyFunction, public ScreenFunction {
#ifdef SWIG
    %mixin RubyScreenFunction "Kahakai::ScreenFunction";
#endif
public:
    RubyScreenFunction(VALUE meth): RubyFunction(meth) {};

            virtual void operator()(KaScreen* scr, 
                                    XEvent* event, 
                                    KaAction* action) {
                VALUE args = rb_ary_new3(3, 
                                         K_SWIG_WRAPPER(scr,    KaScreen),
                                         K_SWIG_WRAPPER(event,  XEvent), 
                                         K_SWIG_WRAPPER(action, KaAction));
                call(args);
    }
};

class RubyWindowFunction: public RubyFunction, public WindowFunction {
#ifdef SWIG
    %mixin RubyWindowFunction "Kahakai::WindowFunction";
#endif
public:
            RubyWindowFunction(VALUE meth): RubyFunction(meth) {};

    virtual void operator()(KaWindow * win, 
                                    XEvent* event, 
                                    KaAction* action) {
                VALUE args = rb_ary_new3(3, 
                                         K_SWIG_WRAPPER(win, KaWindow),
                                         K_SWIG_WRAPPER(event, XEvent),
                                         K_SWIG_WRAPPER(action, KaAction));
                call(args);
    }
};

class RubySimpleFunction: public RubyFunction, public SimpleFunction {
#ifdef SWIG
    %mixin RubySimpleFunction "Kahakai::SimpleFunction";
#endif
public:
    RubySimpleFunction(VALUE meth): RubyFunction(meth) {};

    virtual void operator()(XEvent* event, KaAction* action) {
                VALUE args = rb_ary_new3(2, 
                                         K_SWIG_WRAPPER(event,  XEvent),
                                         K_SWIG_WRAPPER(action, KaAction));
                call(args);
    }
};

RubyMenuFunction* createMenuFunction(VALUE rb_meth) {
  return new RubyMenuFunction(rb_meth);
}

RubyWindowFunction* createWindowFunction(VALUE rb_meth) {
  return new RubyWindowFunction(rb_meth);
}

RubyScreenFunction* createScreenFunction(VALUE rb_meth) {
  return new RubyScreenFunction(rb_meth);
}

RubySimpleFunction* createSimpleFunction(VALUE rb_meth) {
  return new RubySimpleFunction(rb_meth);
}
